@@ -2,20 +2,19 @@ import pkg_resources
 
 # exports
 from imod import (
+    couplers,
     data,
     evaluate,
     flow,
-    gen,
-    idf,
-    ipf,
     mf6,
+    msw,
     prepare,
-    rasterio,
     select,
     util,
     visualize,
     wq,
 )
+from imod.formats import gen, idf, ipf, prj, rasterio
 
 # version
 try:
