@@ -13,8 +13,10 @@ from imod.flow.cap import MetaSwap
 from imod.flow.chd import ConstantHead
 from imod.flow.conductivity import (
     HorizontalHydraulicConductivity,
+    Transmissivity,
     VerticalAnisotropy,
     VerticalHydraulicConductivity,
+    VerticalResistance,
 )
 from imod.flow.dis import TimeDiscretization
 from imod.flow.drn import Drain
@@ -22,6 +24,7 @@ from imod.flow.evt import EvapoTranspiration
 from imod.flow.ghb import GeneralHeadBoundary
 from imod.flow.hfb import HorizontalFlowBarrier
 from imod.flow.model import ImodflowModel
+from imod.flow.oc import OutputControl
 from imod.flow.rch import Recharge
 from imod.flow.riv import River
 from imod.flow.slv import PreconditionedConjugateGradientSolver
