@@ -85,14 +85,15 @@ class Model(collections.UserDict):
 
 class ImodflowModel(Model):
     """
-    iMODFLOW model.
+    Class representing iMODFLOW model input. Running it requires iMOD5.
+
+    `Download iMOD5 here <https://oss.deltares.nl/web/imod/download-imod5>`_
 
     Attributes
     ----------
-    modelname : str
-    check : str, optional
-        When to perform model checks {None, "defer", "eager"}.
-        Defaults to "defer".
+    modelname : str check : str, optional
+        When to perform model checks {None, "defer", "eager"}. Defaults to
+        "defer".
 
     Examples
     --------
@@ -100,7 +101,7 @@ class ImodflowModel(Model):
     >>> m = Imodflow("example")
     >>> m["riv"] = River(...)
     >>> # ...etc.
-    >>> m.time_discretization(endtime)
+    >>> m.create_time_discretization(endtime)
     >>> m.write()
     """
 
@@ -175,6 +176,14 @@ class ImodflowModel(Model):
                 raise ValueError("Use either cftime or numpy.datetime64[ns].")
 
     def time_discretization(self, times):
+        warnings.warn(
+            f"{self.__class__.__name__}.time_discretization() is deprecated. "
+            f"In the future call {self.__class__.__name__}.create_time_discretization().",
+            DeprecationWarning,
+        )
+        self.create_time_discretization(additional_times=times)
+
+    def create_time_discretization(self, additional_times):
         """
         Collect all unique times from model packages and additional given `times`. These
         unique times are used as stress periods in the model. All stress packages must
@@ -193,13 +202,13 @@ class ImodflowModel(Model):
         - Every stress has to be defined on the first stress period (this is a
           modflow requirement)
 
-        Or visually (every letter a date in the time axes)::
+        Or visually (every letter a date in the time axes):
 
-            recharge a - b - c - d - e - f
-            river    g - - - - h - - - - j
-            times    - - - - - - - - - - - i
+        >>> recharge a - b - c - d - e - f
+        >>> river    g - - - - h - - - - j
+        >>> times    - - - - - - - - - - - i
+        >>> model    a - b - c h d - e - f i
 
-            model    a - b - c h d - e - f i
 
         with the stress periods defined between these dates. I.e. the model times are the set of all times you include in the model.
 
@@ -214,28 +223,30 @@ class ImodflowModel(Model):
         --------
         Add a single time:
 
-        >>> m.time_discretization("2001-01-01")
+        >>> m.create_time_discretization("2001-01-01")
 
         Add a daterange:
 
-        >>> m.time_discretization(pd.daterange("2000-01-01", "2001-01-01"))
+        >>> m.create_time_discretization(pd.daterange("2000-01-01", "2001-01-01"))
 
         Add a list of times:
 
-        >>> m.time_discretization(["2000-01-01", "2001-01-01"])
+        >>> m.create_time_discretization(["2000-01-01", "2001-01-01"])
 
         """
 
         # Make sure it's an iterable
-        if not isinstance(times, (np.ndarray, list, tuple, pd.DatetimeIndex)):
-            times = [times]
+        if not isinstance(
+            additional_times, (np.ndarray, list, tuple, pd.DatetimeIndex)
+        ):
+            additional_times = [additional_times]
 
         # Loop through all packages, check if cftime is required.
         self.use_cftime = self._use_cftime()
         # use_cftime is None if you no datetimes are present in packages
         # use_cftime is False if np.datetimes present in packages
         # use_cftime is True if cftime.datetime present in packages
-        for time in times:
+        for time in additional_times:
             if issubclass(type(time), cftime.datetime):
                 if self.use_cftime is None:
                     self.use_cftime = True
@@ -247,7 +258,9 @@ class ImodflowModel(Model):
         if self.use_cftime is None:
             self.use_cftime = False
 
-        times = [timeutil.to_datetime(time, self.use_cftime) for time in times]
+        times = [
+            timeutil.to_datetime(time, self.use_cftime) for time in additional_times
+        ]
         times, first_times = insert_unique_package_times(self.items(), times)
 
         # Check if every transient package commences at the same time.
@@ -395,8 +408,11 @@ class ImodflowModel(Model):
         # These are the strings atop each stress period in the projectfile
         times_composed.update({key: key for key in periods_composed.keys()})
 
+        # Add steady-state for packages without time specified
+        times_composed["steady-state"] = "steady-state"
+
         rendered = []
-        ignored = ["dis"]
+        ignored = ["dis", "oc"]
 
         for key, package in self.items():
             pkg_id = package._pkg_id
@@ -491,7 +507,7 @@ class ImodflowModel(Model):
         directory=pathlib.Path("."),
         result_dir=None,
         resultdir_is_workdir=False,
-        render_projectfile=True,
+        convert_to="mf2005_namfile",
     ):
         """
         Writes model input files.
@@ -504,14 +520,16 @@ class ImodflowModel(Model):
         result_dir : str, pathlib.Path
             Path to directory in which output will be written when running the
             model. Is written as the value of the ``result_dir`` key in the
-            runfile.
-
-            See the examples.
+            runfile. See the examples.
         resultdir_is_workdir: boolean, optional
             Wether the set all input paths in the runfile relative to the output
-            directory. Because iMOD-wq generates a number of files in its working
-            directory, it may be advantageous to set the working directory to
-            a different path than the runfile location.
+            directory. Because iMOD-wq generates a number of files in its
+            working directory, it may be advantageous to set the working
+            directory to a different path than the runfile location.
+        convert_to: str
+            The type of object to convert the projectfile to in the
+            configuration ini file. Should be one of ``["mf2005_namfile",
+            "mf6_namfile", "runfile"]``.
 
         Returns
         -------
@@ -525,9 +543,31 @@ class ImodflowModel(Model):
 
         >>> model.write(directory="input", result_dir="output")
 
-        And in the runfile, a value of ``../../output`` will be written for
-        result_dir.
+        And in the ``config_run.ini``, a value of ``../../output`` will be
+        written for ``result_dir``. This ``config_run.ini`` has to be called
+        with iMOD 5 to convert the model projectfile to a Modflow 2005 namfile.
+        To specify a conversion to a runfile, run:
+
+        >>> model.write(directory="input", convert_to="runfile")
+
+        You can then run the following command to convert the projectfile to a runfile:
+
+        >>> path/to/iMOD5.exe ./input/config_run.ini
+
+        `Download iMOD5 here <https://oss.deltares.nl/web/imod/download-imod5>`_
+
         """
+        directory = pathlib.Path(directory)
+
+        allowed_conversion_settings = ["mf2005_namfile", "mf6_namfile", "runfile"]
+        if convert_to not in allowed_conversion_settings:
+            raise ValueError(
+                f"Got convert_setting: '{convert_to}', should be one of: {allowed_conversion_settings}"
+            )
+
+        # Currently only supported, no runfile can be directly written by iMOD Python
+        # TODO: Add runfile support
+        render_projectfile = True
 
         # TODO: Find a cleaner way to pack and unpack these paths
         (
@@ -572,13 +612,37 @@ class ImodflowModel(Model):
         self[diskey].save(time_path)
 
         # Create and write INI file to configure conversion/simulation
+        ockey = self._get_pkgkey("oc")
+        bndkey = self._get_pkgkey("bnd")
+        nlayer = self[bndkey]["layer"].size
+
+        if ockey is None:
+            raise ValueError("No OutputControl was specified for the model")
+        else:
+            oc_configuration = self[ockey]._compose_oc_configuration(nlayer)
+
+        outfilepath = directory / runfilepath
+
+        RUNFILE_OPTIONS = {
+            "mf2005_namfile": dict(
+                sim_type=2, namfile_out=outfilepath.with_suffix(".nam")
+            ),
+            "runfile": dict(sim_type=1, runfile_out=outfilepath.with_suffix(".run")),
+            "mf6_namfile": dict(
+                sim_type=3, namfile_out=outfilepath.with_suffix(".nam")
+            ),
+        }
+
+        conversion_settings = RUNFILE_OPTIONS[convert_to]
+
         config = IniFile(
-            sim_type=2,
             function="runfile",
             prjfile_in=directory / runfilepath.name,
-            namfile_out=directory / (runfilepath.stem + ".nam"),
             iss=1,
             timfname=directory / time_path.name,
+            output_folder=result_dir,
+            **conversion_settings,
+            **oc_configuration,
         )
         config_content = config.render()
 
@@ -593,9 +657,9 @@ class ImodflowModel(Model):
                 try:
                     pkg.save(directory=directory / pkgname)
                 except Exception as error:
-                    raise RuntimeError(
-                        f"An error occured during saving of package: {pkgname}."
-                    ) from error
+                    raise type(error)(
+                        f"{error}/nAn error occured during saving of package: {pkgname}."
+                    )
 
     def _check_top_bottom(self):
         """Check whether bottom of a layer does not exceed a top somewhere."""
