@@ -271,10 +271,12 @@ class Package(
                     d["layer"] = layer
                     values[layer] = self._compose_path(d, pattern=pattern)
                 else:
+                    dim_sel = {}
                     if "layer" in da.dims:
-                        values[layer] = da.sel(layer=layer).values[()]
-                    else:
-                        values[layer] = da.values[()]
+                        dim_sel["layer"] = layer
+                    if "time" in da.dims:
+                        dim_sel["time"] = time
+                    values[layer] = da.sel(**dim_sel).values[()]
 
         return values
 
@@ -623,7 +625,7 @@ class BoundaryCondition(Package, abc.ABC):
                 varname, directory, nlayer, time=None
             )
             values.append(composed_layers)
-            keys.append([self._pkg_id, "1", varname, system_index])
+            keys.append([self._pkg_id, "steady-state", varname, system_index])
 
         return keys, values
 
@@ -670,9 +672,9 @@ class TopBoundaryCondition(BoundaryCondition, abc.ABC):
         composition_first_layer = util.initialize_nested_dict(5)
 
         # Loop over nested dict, it is not pretty
-        for (a, aa) in composition[self._pkg_id].items():
-            for (b, bb) in aa.items():
-                for (c, cc) in bb.items():
+        for a, aa in composition[self._pkg_id].items():
+            for b, bb in aa.items():
+                for c, cc in bb.items():
                     composition_first_layer[a][b][c][1] = cc[1]
         return composition_first_layer
 
@@ -682,7 +684,6 @@ class TopBoundaryCondition(BoundaryCondition, abc.ABC):
         globaltimes,
         nlayer,
     ):
-
         composition = super().compose(
             directory,
             globaltimes,
