@@ -1,4 +1,8 @@
+import numpy as np
+import xarray as xr
+
 from imod.mf6.pkgbase import Package
+from imod.schemata import DTypeSchema
 
 
 class Solution(Package):
@@ -17,6 +21,9 @@ class Solution(Package):
 
     Parameters
     ----------
+    modelnames: list of str
+        Which models to solve in this solution. Only models of the same type
+        (GWF or GWT) should be added to the same solution.
     outer_dvclose: float
         real value defining the head change criterion for convergence of the
         outer (nonlinear) iterations, in units of length. When the maximum
@@ -329,14 +336,36 @@ class Solution(Package):
         SolutionPresetSimple: No Default
         SolutionPresetModerate: No Default
         SolutionPresetComplex: No Default
+    validate: {True, False}
+        Flag to indicate whether the package should be validated upon
+        initialization. This raises a ValidationError if package input is
+        provided in the wrong manner. Defaults to True.
     """
 
     _pkg_id = "ims"
     _keyword_map = {}
+
+    _init_schemata = {
+        "outer_dvclose": [DTypeSchema(np.floating)],
+        "outer_maximum": [DTypeSchema(np.integer)],
+        "inner_maximum": [DTypeSchema(np.integer)],
+        "inner_dvclose": [DTypeSchema(np.floating)],
+        "inner_rclose": [DTypeSchema(np.floating)],
+        "under_relaxation_theta": [DTypeSchema(np.floating)],
+        "under_relaxation_kappa": [DTypeSchema(np.floating)],
+        "under_relaxation_gamma": [DTypeSchema(np.floating)],
+        "under_relaxation_momentum": [DTypeSchema(np.floating)],
+        "backtracking_number": [DTypeSchema(np.integer)],
+        "backtracking_tolerance": [DTypeSchema(np.floating)],
+        "backtracking_reduction_factor": [DTypeSchema(np.floating)],
+        "backtracking_residual_limit": [DTypeSchema(np.floating)],
+        "number_orthogonalizations": [DTypeSchema(np.integer)],
+    }
     _template = Package._initialize_template(_pkg_id)
 
     def __init__(
         self,
+        modelnames,
         outer_dvclose,
         outer_maximum,
         inner_maximum,
@@ -362,8 +391,15 @@ class Solution(Package):
         print_option="summary",
         csv_output=False,
         no_ptc=False,
+        validate: bool = True,
     ):
         super().__init__()
+        self.dataset = xr.Dataset()
+        # Make sure the modelnames are set as a variable rather than dimension:
+        if isinstance(modelnames, xr.DataArray):
+            self.dataset["modelnames"] = modelnames
+        else:
+            self.dataset["modelnames"] = ("model", modelnames)
         self.dataset["outer_dvclose"] = outer_dvclose
         self.dataset["outer_maximum"] = outer_maximum
         self.dataset["under_relaxation"] = under_relaxation
@@ -389,10 +425,14 @@ class Solution(Package):
         self.dataset["print_option"] = print_option
         self.dataset["csv_output"] = csv_output
         self.dataset["no_ptc"] = no_ptc
+        self._validate_init_schemata(validate)
 
 
-def SolutionPresetSimple(print_option, csv_output, no_ptc):
+def SolutionPresetSimple(
+    modelnames, print_option="summary", csv_output=False, no_ptc=False
+):
     solution = Solution(
+        modelnames=modelnames,
         print_option=print_option,
         csv_output=csv_output,
         no_ptc=no_ptc,
@@ -422,8 +462,11 @@ def SolutionPresetSimple(print_option, csv_output, no_ptc):
     return solution
 
 
-def SolutionPresetModerate(print_option, csv_output, no_ptc):
+def SolutionPresetModerate(
+    modelnames, print_option="summary", csv_output=False, no_ptc=False
+):
     solution = Solution(
+        modelnames=modelnames,
         print_option=print_option,
         csv_output=csv_output,
         no_ptc=no_ptc,
@@ -453,8 +496,11 @@ def SolutionPresetModerate(print_option, csv_output, no_ptc):
     return solution
 
 
-def SolutionPresetComplex(print_option, csv_output, no_ptc):
+def SolutionPresetComplex(
+    modelnames, print_option="summary", csv_output=False, no_ptc=False
+):
     solution = Solution(
+        modelnames=modelnames,
         print_option=print_option,
         csv_output=csv_output,
         no_ptc=no_ptc,
