@@ -1,11 +1,20 @@
 import abc
+import numbers
 import pathlib
+from collections import defaultdict
+from typing import Dict, List
 
 import jinja2
 import numpy as np
 import xarray as xr
 import xugrid as xu
 
+import imod
+from imod.mf6.validation import validation_pkg_error_message
+from imod.schemata import ValidationError
+
+TRANSPORT_PACKAGES = ("adv", "dsp", "ssm", "mst", "ist", "src")
+
 
 def dis_recarr(arrdict, layer, notnull):
     # Define the numpy structured array dtype
@@ -16,7 +25,7 @@ def dis_recarr(arrdict, layer, notnull):
     nrow = notnull.sum()
     recarr = np.empty(nrow, dtype=sparse_dtype)
     # Fill in the indices
-    if layer.size == 1:
+    if notnull.ndim == 2:
         recarr["row"], recarr["column"] = (np.argwhere(notnull) + 1).transpose()
         recarr["layer"] = layer
     else:
@@ -36,7 +45,7 @@ def disv_recarr(arrdict, layer, notnull):
     nrow = notnull.sum()
     recarr = np.empty(nrow, dtype=sparse_dtype)
     # Fill in the indices
-    if layer.size == 1:
+    if notnull.ndim == 1 and layer.size == 1:
         recarr["cell2d"] = (np.argwhere(notnull) + 1).transpose()
         recarr["layer"] = layer
     else:
@@ -95,35 +104,31 @@ class Package(abc.ABC):
 
         Refer to the xarray documentation for the possible keyword arguments.
         """
-
-        # Throw error if user tries to use old functionality
-        if "cache" in kwargs:
-            if kwargs["cache"] is not None:
-                raise NotImplementedError(
-                    "Caching functionality in pkg.from_file() is removed."
-                )
-
         path = pathlib.Path(path)
-
-        # See https://stackoverflow.com/a/2169191
-        # We expect the data in the netcdf has been saved a a package
-        # thus the checks run by __init__ and __setitem__ do not have
-        # to be called again.
-        return_cls = cls.__new__(cls)
-
         if path.suffix in (".zip", ".zarr"):
             # TODO: seems like a bug? Remove str() call if fixed in xarray/zarr
-            return_cls.dataset = xr.open_zarr(str(path), **kwargs)
+            dataset = xr.open_zarr(str(path), **kwargs)
         else:
-            return_cls.dataset = xr.open_dataset(path, **kwargs)
+            dataset = xr.open_dataset(path, **kwargs)
 
-        return return_cls
+        if dataset.ugrid_roles.topology:
+            dataset = xu.UgridDataset(dataset)
+
+        # Replace NaNs by None
+        for key, value in dataset.items():
+            stripped_value = value.values[()]
+            if isinstance(stripped_value, numbers.Real) and np.isnan(stripped_value):
+                dataset[key] = None
+
+        instance = cls.__new__(cls)
+        instance.dataset = dataset
+        return instance
 
     def __init__(self, allargs=None):
         if allargs is not None:
             for arg in allargs.values():
                 if isinstance(arg, xu.UgridDataArray):
-                    self.dataset = xu.UgridDataset(grid=arg.ugrid.grid)
+                    self.dataset = xu.UgridDataset(grids=arg.ugrid.grid)
                     return
         self.dataset = xr.Dataset()
 
@@ -161,6 +166,11 @@ class Package(abc.ABC):
         # Test numpy bool (not singleton)
         elif isinstance(value, np.bool_) and not value:
             return False
+        # When dumping to netCDF and reading back, None will have been
+        # converted into a NaN. Only check NaN if it's a floating type to avoid
+        # TypeErrors.
+        elif np.issubdtype(type(value), np.floating) and np.isnan(value):
+            return False
         else:
             return True
 
@@ -181,6 +191,8 @@ class Package(abc.ABC):
             fname = "sln-ims.j2"
         elif pkg_id == "tdis":
             fname = "sim-tdis.j2"
+        elif pkg_id in TRANSPORT_PACKAGES:
+            fname = f"gwt-{pkg_id}.j2"
         else:
             fname = f"gwf-{pkg_id}.j2"
         return env.get_template(fname)
@@ -221,26 +233,28 @@ class Package(abc.ABC):
 
         return recarr
 
-    def _check_layer_presence(self, ds):
-        """
-        If layer present in coordinates and dimensions return layers,
-        if not return None
-        """
-
-        if "layer" in ds.coords:
-            layer = ds["layer"].values
-        else:
-            layer = None
-        return layer
-
     def _ds_to_arrdict(self, ds):
         arrdict = {}
         for datavar in ds.data_vars:
             if ds[datavar].shape == ():
                 raise ValueError(
-                    f"{datavar} in {ds._pkg_id} package cannot be a scalar"
+                    f"{datavar} in {self._pkg_id} package cannot be a scalar"
                 )
-            arrdict[datavar] = ds[datavar].values
+            auxiliary_vars = (
+                self.get_auxiliary_variable_names()
+            )  # returns something like {"concentration": "species"}
+            if datavar in auxiliary_vars.keys():  # if datavar is concentration
+                if (
+                    auxiliary_vars[datavar] in ds[datavar].dims
+                ):  # if this concentration array has the species dimension
+                    for s in ds[datavar].values:  # loop over species
+                        arrdict[s] = (
+                            ds[datavar]
+                            .sel({auxiliary_vars[datavar]: s})
+                            .values  # store species array under its species name
+                        )
+            else:
+                arrdict[datavar] = ds[datavar].values
         return arrdict
 
     def write_binary_griddata(self, outpath, da, dtype):
@@ -286,14 +300,31 @@ class Package(abc.ABC):
             # "reshape" a long row with "word wrap"; they cannot as easily
             # ignore newlines.
             fmt = self._number_format(dtype)
-            np.savetxt(fname=f, X=da.values.reshape((1, -1)), fmt=fmt)
+            data = da.values
+            if data.ndim > 2:
+                np.savetxt(fname=f, X=da.values.reshape((1, -1)), fmt=fmt)
+            else:
+                np.savetxt(fname=f, X=da.values, fmt=fmt)
 
     def render(self, directory, pkgname, globaltimes, binary):
         d = {}
-        for k, v in self.dataset.data_vars.items():  # pylint:disable=no-member
-            value = v.values[()]
-            if self._valid(value):  # skip None and False
-                d[k] = value
+        if directory is None:
+            pkg_directory = pkgname
+        else:
+            pkg_directory = directory / pkgname
+        for varname in self.dataset.data_vars:
+            key = self._keyword_map.get(varname, varname)
+
+            if hasattr(self, "_grid_data") and varname in self._grid_data:
+                layered, value = self._compose_values(
+                    self.dataset[varname], pkg_directory, key, binary=binary
+                )
+                if self._valid(value):  # skip False or None
+                    d[f"{key}_layered"], d[key] = layered, value
+            else:
+                value = self[varname].values[()]
+                if self._valid(value):  # skip False or None
+                    d[key] = value
         return self._template.render(d)
 
     @staticmethod
@@ -328,7 +359,6 @@ class Package(abc.ABC):
                 path = (directory / f"{name}.dat").as_posix()
                 values.append(f"open/close {path}")
         else:
-
             if "layer" in da.dims:
                 layered = True
                 for layer in da.coords["layer"]:
@@ -344,7 +374,6 @@ class Package(abc.ABC):
 
     def write(self, directory, pkgname, globaltimes, binary):
         directory = pathlib.Path(directory)
-
         self.write_blockfile(directory, pkgname, globaltimes, binary=binary)
 
         if hasattr(self, "_grid_data"):
@@ -362,6 +391,34 @@ class Package(abc.ABC):
                             path = pkgdirectory / f"{key}.dat"
                             self.write_text_griddata(path, da, dtype)
 
+    def _validate(self, schemata: Dict, **kwargs) -> Dict[str, List[ValidationError]]:
+        errors = defaultdict(list)
+        for variable, var_schemata in schemata.items():
+            for schema in var_schemata:
+                if (
+                    variable in self.dataset.keys()
+                ):  # concentration only added to dataset if specified
+                    try:
+                        schema.validate(self.dataset[variable], **kwargs)
+                    except ValidationError as e:
+                        errors[variable].append(e)
+        return errors
+
+    def _validate_init_schemata(self, validate: bool):
+        """
+        Run the "cheap" schema validations.
+
+        The expensive validations are run during writing. Some are only
+        available then: e.g. idomain to determine active part of domain.
+        """
+        if not validate:
+            return
+        errors = self._validate(self._init_schemata)
+        if len(errors) > 0:
+            message = validation_pkg_error_message(errors)
+            raise ValidationError(message)
+        return
+
     def _netcdf_path(self, directory, pkgname):
         """create path for netcdf, this function is also used to create paths to use inside the qgis projectfiles"""
         return directory / pkgname / f"{self._pkg_id}.nc"
@@ -411,6 +468,51 @@ class Package(abc.ABC):
         spatial_ds.to_netcdf(path)
         return has_dims
 
+    def _get_vars_to_check(self):
+        """
+        Helper function to get all variables which were not set to None
+        """
+        variables = []
+        for var in self._metadata_dict.keys():
+            if (  # Filter optional variables not filled in
+                self.dataset[var].size != 1
+            ) or (
+                self.dataset[var] != None  # noqa: E711
+            ):
+                variables.append(var)
+
+        return variables
+
+    def period_data(self):
+        result = []
+        if hasattr(self, "_period_data"):
+            result += self._period_data
+        if hasattr(self, "_auxiliary_data"):
+            for aux_var_name, aux_var_dimensions in self._auxiliary_data.items():
+                if aux_var_name in self.dataset.keys():
+                    for s in (
+                        self.dataset[aux_var_name].coords[aux_var_dimensions].values
+                    ):
+                        result.append(s)
+        return result
+
+    def add_periodic_auxiliary_variable(self):
+        if hasattr(self, "_auxiliary_data"):
+            for aux_var_name, aux_var_dimensions in self._auxiliary_data.items():
+                aux_coords = (
+                    self.dataset[aux_var_name].coords[aux_var_dimensions].values
+                )
+                for s in aux_coords:
+                    self.dataset[s] = self.dataset[aux_var_name].sel(
+                        {aux_var_dimensions: s}
+                    )
+
+    def get_auxiliary_variable_names(self):
+        result = {}
+        if hasattr(self, "_auxiliary_data"):
+            result.update(self._auxiliary_data)
+        return result
+
 
 class BoundaryCondition(Package, abc.ABC):
     """
@@ -422,12 +524,33 @@ class BoundaryCondition(Package, abc.ABC):
     not the array input which is used in :class:`Package`.
     """
 
+    def set_repeat_stress(self, times) -> None:
+        """
+        Set repeat stresses: re-use data of earlier periods.
+
+        Parameters
+        ----------
+        times: Dict of datetime-like to datetime-like.
+            The data of the value datetime is used for the key datetime.
+        """
+        keys = [
+            imod.wq.timeutil.to_datetime(key, use_cftime=False) for key in times.keys()
+        ]
+        values = [
+            imod.wq.timeutil.to_datetime(value, use_cftime=False)
+            for value in times.values()
+        ]
+        self.dataset["repeat_stress"] = xr.DataArray(
+            data=np.column_stack((keys, values)),
+            dims=("repeat", "repeat_items"),
+        )
+
     def _max_active_n(self):
         """
         Determine the maximum active number of cells that are active
         during a stress period.
         """
-        da = self.dataset[self._period_data[0]]
+        da = self.dataset[self.period_data()[0]]
         if "time" in da.coords:
             nmax = int(da.groupby("time").count(xr.ALL_DIMS).max())
         else:
@@ -449,7 +572,7 @@ class BoundaryCondition(Package, abc.ABC):
         """
         Writes a modflow6 binary data file
         """
-        layer = self._check_layer_presence(ds)
+        layer = ds["layer"].values
         arrdict = self._ds_to_arrdict(ds)
         sparse_data = self.to_sparse(arrdict, layer)
         outpath.parent.mkdir(exist_ok=True, parents=True)
@@ -468,17 +591,29 @@ class BoundaryCondition(Package, abc.ABC):
         if "time" in bin_ds:  # one of bin_ds has time
             package_times = bin_ds.coords["time"].values
             starts = np.searchsorted(globaltimes, package_times) + 1
-            for i, s in enumerate(starts):
+            for i, start in enumerate(starts):
                 path = directory / pkgname / f"{self._pkg_id}-{i}.{ext}"
-                periods[s] = path.as_posix()
+                periods[start] = path.as_posix()
+
+            repeat_stress = self.dataset.get("repeat_stress")
+            if repeat_stress is not None and repeat_stress.values[()] is not None:
+                keys = repeat_stress.isel(repeat_items=0).values
+                values = repeat_stress.isel(repeat_items=1).values
+                repeat_starts = np.searchsorted(globaltimes, keys) + 1
+                values_index = np.searchsorted(globaltimes, values) + 1
+                for i, start in zip(values_index, repeat_starts):
+                    periods[start] = periods[i]
+                # Now make sure the periods are sorted by key.
+                periods = dict(sorted(periods.items()))
         else:
             path = directory / pkgname / f"{self._pkg_id}.{ext}"
             periods[1] = path.as_posix()
+
         return periods
 
     def get_options(self, d, not_options=None):
         if not_options is None:
-            not_options = self._period_data
+            not_options = self.period_data()
 
         for varname in self.dataset.data_vars.keys():  # pylint:disable=no-member
             if varname in not_options:
@@ -491,17 +626,39 @@ class BoundaryCondition(Package, abc.ABC):
     def render(self, directory, pkgname, globaltimes, binary):
         """Render fills in the template only, doesn't write binary data"""
         d = {"binary": binary}
-        bin_ds = self[list(self._period_data)]
+        bin_ds = self[self.period_data()]
         d["periods"] = self.period_paths(
             directory, pkgname, globaltimes, bin_ds, binary
         )
         # construct the rest (dict for render)
         d = self.get_options(d)
         d["maxbound"] = self._max_active_n()
+
+        # now we should add the auxiliary variable names to d
+        auxiliaries = (
+            self.get_auxiliary_variable_names()
+        )  # returns someting like {"concentration": "species"}
+
+        # loop over the types of auxiliary variables (for example concentration)
+        for auxvar in auxiliaries.keys():
+            # if "concentration" is a variable of this dataset
+            if auxvar in self.dataset.data_vars:
+                # if our concentration dataset has the species coordinate
+                if auxiliaries[auxvar] in self.dataset[auxvar].coords:
+                    # assign the species names list to d
+                    d["auxiliary"] = self.dataset[auxiliaries[auxvar]].values
+                else:
+                    # the error message is more specific than the code at this point.
+                    raise ValueError(
+                        f"{auxvar} requires a {auxiliaries[auxvar]} coordinate."
+                    )
+
         return self._template.render(d)
 
     def write_perioddata(self, directory, pkgname, binary):
-        bin_ds = self[list(self._period_data)]
+        if len(self.period_data()) == 0:
+            return
+        bin_ds = self[self.period_data()]
 
         if binary:
             ext = "bin"
@@ -537,6 +694,22 @@ class BoundaryCondition(Package, abc.ABC):
             binary=binary,
         )
 
+    def assign_dims(self, arg) -> Dict:
+        is_da = isinstance(arg, xr.DataArray)
+        if is_da and "time" in arg.coords:
+            if arg.ndim != 2:
+                raise ValueError("time varying variable: must be 2d")
+            if arg.dims[0] != "time":
+                arg = arg.transpose()
+            da = xr.DataArray(
+                data=arg.values, coords={"time": arg["time"]}, dims=["time", "index"]
+            )
+            return da
+        elif is_da:
+            return ("index", arg.values)
+        else:
+            return ("index", arg)
+
 
 class AdvancedBoundaryCondition(BoundaryCondition, abc.ABC):
     """
@@ -584,3 +757,37 @@ class AdvancedBoundaryCondition(BoundaryCondition, abc.ABC):
         self.write_blockfile(directory, pkgname, globaltimes, binary=False)
         self.write_perioddata(directory, pkgname, binary=False)
         self.write_packagedata(directory, pkgname, binary=False)
+
+
+class DisStructuredBoundaryCondition(BoundaryCondition):
+    def to_sparse(self, arrdict, layer):
+        spec = []
+        for key in arrdict:
+            if key in ["layer", "row", "column"]:
+                spec.append((key, np.int32))
+            else:
+                spec.append((key, np.float64))
+
+        sparse_dtype = np.dtype(spec)
+        nrow = next(iter(arrdict.values())).size
+        recarr = np.empty(nrow, dtype=sparse_dtype)
+        for key, arr in arrdict.items():
+            recarr[key] = arr
+        return recarr
+
+
+class DisVerticesBoundaryCondition(BoundaryCondition):
+    def to_sparse(self, arrdict, layer):
+        spec = []
+        for key in arrdict:
+            if key in ["layer", "cell2d"]:
+                spec.append((key, np.int32))
+            else:
+                spec.append((key, np.float64))
+
+        sparse_dtype = np.dtype(spec)
+        nrow = next(iter(arrdict.values())).size
+        recarr = np.empty(nrow, dtype=sparse_dtype)
+        for key, arr in arrdict.items():
+            recarr[key] = arr
+        return recarr
