@@ -293,10 +293,10 @@ def _check_monotonic(dxs, dim):
 def _set_cellsizes(da, dims):
     for dim in dims:
         dx_string = f"d{dim}"
-        if dx_string not in da:
-            dx, _, _ = imod.util.coord_reference(da[dim])
+        if dx_string not in da.coords:
+            dx, _, _ = imod.util.coord_reference(da.coords[dim])
             if isinstance(dx, (int, float)):
-                dx = np.full(da[dim].size, dx)
+                dx = np.full(da.coords[dim].size, dx)
             da = da.assign_coords({dx_string: (dim, dx)})
     return da
 
@@ -304,10 +304,15 @@ def _set_cellsizes(da, dims):
 def _set_scalar_cellsizes(da):
     for dim in da.dims:
         dx_string = f"d{dim}"
-        if dx_string in da:
-            dx = da[dx_string]
-            if np.allclose(dx, dx[0]):
-                da = da.assign_coords({dx_string: dx[0]})
+        if dx_string in da.coords:
+            dx = da.coords[dx_string]
+            # Ensure no leftover coordinates in scalar
+            if dx.ndim == 0:  # Catch case where dx already is a scalar
+                dx_scalar = dx.values[()]
+            else:
+                dx_scalar = dx.values[0]
+            if np.allclose(dx, dx_scalar):
+                da = da.assign_coords({dx_string: dx_scalar})
     return da
 
 
@@ -411,7 +416,7 @@ def _define_slices(src, like):
     return dst_expanded_slices, dst_chunks_shape
 
 
-def _sel_chunks(da, expanded_slices):
+def _sel_chunks(da, dims, expanded_slices):
     """
     Using the slices created with the functions above, use xarray's index
     selection methods to create a list of "like" DataArrays which are used
@@ -423,7 +428,7 @@ def _sel_chunks(da, expanded_slices):
     das = []
     for dim_slices in expanded_slices:
         slice_dict = {}
-        for dim, dim_slice in zip(da.dims, dim_slices):
+        for dim, dim_slice in zip(dims, dim_slices):
             slice_dict[dim] = dim_slice
         das.append(da.isel(**slice_dict))
     return das
@@ -496,22 +501,21 @@ def geometric_mean(values, weights):
     if normsum == 0:
         return np.nan
 
-    m = 0
     for i in range(values.size):
         w = weights[i] / normsum
         v = values[i]
-        if np.isnan(v):
-            continue
-        if w > 0:
+        # Skip if v == 0, v is NaN or w == 0 (no contribution)
+        if v > 0 and w > 0:
             v_agg += w * np.log(abs(v))
             w_sum += w
-            if v < 0:
-                m += 1
+        # Do not reduce over negative values: would require complex numbers.
+        elif v < 0:
+            return np.nan
 
     if w_sum == 0:
         return np.nan
     else:
-        return (-1.0) ** m * np.exp((1.0 / w_sum) * v_agg)
+        return np.exp((1.0 / w_sum) * v_agg)
 
 
 def sum(values, weights):
