@@ -279,7 +279,7 @@ def polygonize(da):
 
     geometries = []
     colvalues = []
-    for (geom, colval) in shapes:
+    for geom, colval in shapes:
         geometries.append(sg.Polygon(geom["coordinates"][0]))
         colvalues.append(colval)
 
@@ -972,10 +972,10 @@ def zonal_aggregate_raster(
         of aggregation.
     column : str
         column name of path, integer IDs defining zones.
-    resolution : float
-        cellsize at which the rasterization of polygons and sampling occurs
     raster : xarray.DataArray
         Raster data from which to sample and aggregate data
+    resolution : float
+        cellsize at which the rasterization of polygons and sampling occurs
     method : Union[str, Callable]
         Aggregation method.
         Anything that is acceptable by a pandas groupby aggregate:
@@ -994,14 +994,14 @@ def zonal_aggregate_raster(
 
     >>> import imod
     >>> surface_level = imod.rasterio.open("surface_level.tif")
-    >>> df = imod.prepare.spatial.overlay_raster_intersection(
+    >>> df = imod.prepare.spatial.zonal_aggregate_raster(
     >>>    "water-bodies.shp", "id", 1.0, surface_level, "mean"
     >>> )
 
     For some functions, like the mode, a function should be passed instead:
 
     >>> import pandas as pd
-    >>> df = imod.prepare.spatial.overlay_raster_intersection(
+    >>> df = imod.prepare.spatial.zonal_aggregate_raster(
     >>>    "water-bodies.shp", "id", 1.0, surface_level, pd.Series.mode
     >>> )
     """
@@ -1050,15 +1050,15 @@ def zonal_aggregate_polygons(
         column name of path_a. Defines zones of aggregation.
     column_b : str
         column name of path_b. Data to aggregate.
+    like : xarray.DataArray with dims ("y", "x")
+        Example DataArray of where the cells will be located. Used only for its
+        x and y coordinates.
     resolution : float
         cellsize at which the rasterization, sampling, and area measurement occur.
     method: Union[str, Callable]
         Aggregation method.
         Anything that is acceptable by a pandas groupby aggregate:
         https://pandas.pydata.org/docs/reference/api/pandas.core.groupby.DataFrameGroupBy.aggregate.html
-    like : xarray.DataArray with dims ("y", "x")
-        Example DataArray of where the cells will be located. Used only for its
-        x and y coordinates.
     chunksize : int, optional
         The size of the chunksize. Used for both x and y dimension.
 
