@@ -515,7 +515,7 @@ def mdal_compliant_ugrid2d(ds: xr.Dataset) -> xr.Dataset:
             for layer in stacked["layer"].values:
                 ds[f"{variable}_layer_{layer}"] = stacked.sel(layer=layer, drop=True)
     if "layer" in ds.coords:
-        ds = ds.drop("layer")
+        ds = ds.drop_vars("layer")
 
     # Find topology variables
     for variable in ds.data_vars:
@@ -535,7 +535,17 @@ def mdal_compliant_ugrid2d(ds: xr.Dataset) -> xr.Dataset:
             # "max_face_nodes_dimension": required
             # "face_coordinates": optional
 
+            node_dim = attrs.get("node_dimension")
             edge_dim = attrs.get("edge_dimension")
+            face_dim = attrs.get("face_dimension")
+
+            # Drop the coordinates on the UGRID dimensions
+            to_drop = []
+            for dim in (node_dim, edge_dim, face_dim):
+                if dim is not None and dim in ds.coords:
+                    to_drop.append(dim)
+            ds = ds.drop_vars(to_drop)
+
             if edge_dim and edge_dim not in ds.dims:
                 attrs.pop("edge_dimension")
 
@@ -548,6 +558,7 @@ def mdal_compliant_ugrid2d(ds: xr.Dataset) -> xr.Dataset:
                 attrs.pop("edge_node_connectivity")
 
     # Make sure time is encoded as a float for MDAL
+    # TODO: MDAL requires all data variables to be float (this excludes the UGRID topology data)
     for var in ds.coords:
         if np.issubdtype(ds[var].dtype, np.datetime64):
             ds[var].encoding["dtype"] = np.float64
@@ -581,7 +592,7 @@ def to_ugrid2d(data: Union[xr.DataArray, xr.Dataset]) -> xr.Dataset:
         raise TypeError("data must be xarray.DataArray or xr.Dataset")
 
     grid = xugrid.Ugrid2d.from_structured(data)
-    ds = grid.dataset
+    ds = grid.to_dataset()
 
     if isinstance(data, xr.Dataset):
         for variable in data.data_vars:
@@ -991,6 +1002,88 @@ def where(condition, if_true, if_false, keep_nan: bool = True) -> xr.DataArray:
     return new
 
 
+def replace(da: xr.DataArray, to_replace: Any, value: Any) -> xr.DataArray:
+    """
+    Replace values given in `to_replace` by `value`.
+
+    Parameters
+    ----------
+    da: xr.DataArray
+    to_replace: scalar or 1D array like
+        Which values to replace. If to_replace and value are both array like,
+        they must be the same length.
+    value: scalar or 1D array like
+        Value to replace any values matching `to_replace` with.
+
+    Returns
+    -------
+    xr.DataArray
+        DataArray after replacement.
+
+    Examples
+    --------
+
+    Replace values of 1.0 by 10.0, and 2.0 by 20.0:
+
+    >>> da = xr.DataArray([0.0, 1.0, 1.0, 2.0, 2.0])
+    >>> replaced = imod.util.replace(da, to_replace=[1.0, 2.0], value=[10.0, 20.0])
+
+    """
+    from xarray.core.utils import is_scalar
+
+    def _replace(
+        a: np.ndarray, to_replace: np.ndarray, value: np.ndarray
+    ) -> np.ndarray:
+        flat = da.values.ravel()
+
+        sorter = np.argsort(to_replace)
+        insertion = np.searchsorted(to_replace, flat, sorter=sorter)
+        indices = np.take(sorter, insertion, mode="clip")
+        replaceable = to_replace[indices] == flat
+
+        out = flat.copy()
+        out[replaceable] = value[indices[replaceable]]
+        return out.reshape(a.shape)
+
+    if is_scalar(to_replace):
+        if not is_scalar(value):
+            raise TypeError("if to_replace is scalar, then value must be a scalar")
+        if np.isnan(to_replace):
+            return da.fillna(value)
+        else:
+            return da.where(da != to_replace, other=value)
+    else:
+        to_replace = np.asarray(to_replace)
+        if to_replace.ndim != 1:
+            raise ValueError("to_replace must be 1D or scalar")
+        if is_scalar(value):
+            value = np.full_like(to_replace, value)
+        else:
+            value = np.asarray(value)
+            if to_replace.shape != value.shape:
+                raise ValueError(
+                    f"Replacement arrays must match in shape. "
+                    f"Expecting {to_replace.shape} got {value.shape} "
+                )
+
+    _, counts = np.unique(to_replace, return_counts=True)
+    if (counts > 1).any():
+        raise ValueError("to_replace contains duplicates")
+
+    isnan = np.isnan(to_replace)
+    if isnan.any():
+        i = np.nonzero(isnan)[0]
+        da = da.fillna(value[i])
+
+    return xr.apply_ufunc(
+        _replace,
+        da,
+        kwargs={"to_replace": to_replace, "value": value},
+        dask="parallelized",
+        output_dtypes=[da.dtype],
+    )
+
+
 class MissingOptionalModule:
     """
     Presents a clear error for optional modules.
