@@ -76,11 +76,18 @@ class Model(collections.UserDict):
         selmodel = type(self)(self.modelname, self.check)
         for pkgname, pkg in self.items():
             sel_dims = {k: v for k, v in dimensions.items() if k in pkg.dataset}
+
+            if pkg._pkg_id == "bas6":
+                # da.sel() unsets dimensions for scalars, this messes with bas6
+                # package, because check_ibound is called upon initialization.
+                sel_dims = {
+                    k: [v] if np.isscalar(v) else v for k, v in sel_dims.items()
+                }
             if len(sel_dims) == 0:
                 selmodel[pkgname] = pkg
             else:
                 if pkg._pkg_id != "wel":
-                    selmodel[pkgname].dataset = pkg.dataset.loc[sel_dims]
+                    selmodel[pkgname] = type(pkg)(**pkg.dataset.loc[sel_dims])
                 else:
                     df = pkg.dataset.to_dataframe().drop(columns="save_budget")
                     for k, v in sel_dims.items():
@@ -143,9 +150,7 @@ class Model(collections.UserDict):
                     directory / pattern.format(pkgname=pkgname), **kwargs
                 )
             except Exception as e:
-                raise RuntimeError(
-                    "Package {pkgname} can not be written to NetCDF"
-                ) from e
+                raise type(e)("{e}\nPackage {pkgname} can not be written to NetCDF")
 
     def write_qgis_project(self, directory, crs, filename=None, aggregate_layers=False):
         """
@@ -228,7 +233,7 @@ class SeawatModel(Model):
     >>> m = SeawatModel("example")
     >>> m["riv"] = River(...)
     >>> # ...etc.
-    >>> m.time_discretization(endtime)
+    >>> m.create_time_discretization(endtime)
     >>> m.write()
     """
 
@@ -329,6 +334,14 @@ class SeawatModel(Model):
                 raise ValueError("Use either cftime or numpy.datetime64[ns].")
 
     def time_discretization(self, times):
+        warnings.warn(
+            f"{self.__class__.__name__}.time_discretization() is deprecated. "
+            f"In the future call {self.__class__.__name__}.create_time_discretization().",
+            DeprecationWarning,
+        )
+        self.create_time_discretization(additional_times=times)
+
+    def create_time_discretization(self, additional_times):
         """
         Collect all unique times from model packages and additional given `times`. These
         unique times are used as stress periods in the model. All stress packages must
@@ -343,13 +356,12 @@ class SeawatModel(Model):
         - Additional times can be provided to force shorter stress periods & more detailed output
         - Every stress has to be defined on the first stress period (this is a modflow requirement)
 
-        Or visually (every letter a date in the time axes)::
+        Or visually (every letter a date in the time axes):
 
-            recharge a - b - c - d - e - f
-            river    g - - - - h - - - - j
-            times    - - - - - - - - - - - i
-
-            model    a - b - c h d - e - f i
+        >>> recharge a - b - c - d - e - f
+        >>> river    g - - - - h - - - - j
+        >>> times    - - - - - - - - - - - i
+        >>> model    a - b - c h d - e - f i
 
         with the stress periods defined between these dates. I.e. the model times are the set of all times you include in the model.
 
@@ -360,32 +372,39 @@ class SeawatModel(Model):
             should be given, which will be used as the ending time of the
             simulation.
 
+        Note
+        ----
+        To set the other parameters of the TimeDiscretization object, you have
+        to set these to the object after calling this function.
+
         Examples
         --------
         Add a single time:
 
-        >>> m.time_discretization("2001-01-01")
+        >>> m.create_time_discretization("2001-01-01")
 
         Add a daterange:
 
-        >>> m.time_discretization(pd.daterange("2000-01-01", "2001-01-01"))
+        >>> m.create_time_discretization(pd.daterange("2000-01-01", "2001-01-01"))
 
         Add a list of times:
 
-        >>> m.time_discretization(["2000-01-01", "2001-01-01"])
+        >>> m.create_time_discretization(["2000-01-01", "2001-01-01"])
 
         """
 
         # Make sure it's an iterable
-        if not isinstance(times, (np.ndarray, list, tuple, pd.DatetimeIndex)):
-            times = [times]
+        if not isinstance(
+            additional_times, (np.ndarray, list, tuple, pd.DatetimeIndex)
+        ):
+            additional_times = [additional_times]
 
         # Loop through all packages, check if cftime is required.
         self.use_cftime = self._use_cftime()
         # use_cftime is None if you no datetimes are present in packages
         # use_cftime is False if np.datetimes present in packages
         # use_cftime is True if cftime.datetime present in packages
-        for time in times:
+        for time in additional_times:
             if issubclass(type(time), cftime.datetime):
                 if self.use_cftime is None:
                     self.use_cftime = True
@@ -397,7 +416,9 @@ class SeawatModel(Model):
         if self.use_cftime is None:
             self.use_cftime = False
 
-        times = [timeutil.to_datetime(time, self.use_cftime) for time in times]
+        times = [
+            timeutil.to_datetime(time, self.use_cftime) for time in additional_times
+        ]
         first_times = {}  # first time per package
         for key, pkg in self.items():
             if self._hastime(pkg):
@@ -915,22 +936,23 @@ class SeawatModel(Model):
         # Clip model to extent, set outside extent to nodata or 0
         ml = self.sel(**clip_slices)
         for pck in ml.keys():
-            for d in ml[pck].data_vars:
+            for d in ml[pck].dataset.data_vars:
                 if d in ["ibound", "icbund"]:
                     ml[pck][d] = ml[pck][d].where(extentwithedge == 1, 0)
                 elif "x" in ml[pck][d].dims:
                     ml[pck][d] = ml[pck][d].where(extentwithedge == 1)
 
         # Create boundary conditions as CHD and/or TVC package
-        # Time shifts: assume heads and conc are calculation results. Then:
-        # timestamp of result is _end_ of stress period, while timestamp input
-        # is the _start_ of the stress period.
         if concentration_boundary is not None:
             concentration_boundary = concentration_boundary.sel(**clip_slices)
             concentration_boundary = concentration_boundary.where(edge == 1)
-            concentration_boundary = concentration_boundary.shift(
-                time=-1
-            ).combine_first(concentration_boundary)
+            # Time shifts: assume heads and conc are calculation results. Then:
+            # timestamp of result is _end_ of stress period, while timestamp input
+            # is the _start_ of the stress period.
+            if "time" in concentration_boundary.dims:
+                concentration_boundary = concentration_boundary.shift(
+                    time=-1
+                ).combine_first(concentration_boundary)
 
             ml["tvc"] = imod.wq.TimeVaryingConstantConcentration(
                 concentration=concentration_boundary
@@ -939,7 +961,10 @@ class SeawatModel(Model):
         if heads_boundary is not None:
             heads_boundary = heads_boundary.sel(**clip_slices)
             head_end = heads_boundary.where(edge == 1)
-            head_start = head_end.shift(time=1).combine_first(head_end)
+            if "time" in heads_boundary.dims:
+                head_start = head_end.shift(time=1).combine_first(head_end)
+            else:
+                head_start = head_end
 
             ml["chd"] = imod.wq.ConstantHead(
                 head_start=head_start,
