@@ -174,7 +174,7 @@ class ParallelSolver(Package):
                 self._replace_keyword(d, key)
 
         if self["partition"] == "rcb":
-            d["load_balance_weight"] = self._compose(
+            d["load_balance_weight"] = self._compose_path(
                 {
                     "directory": directory,
                     "name": "load_balance_weight",
